property mySprite, getPDLError, myString, myDelay, myPosition, myDisplaySprite, myHideFlag, myDisplayList, myStartTicks, myDisplayFlag

on getBehaviorDescription me
  return "TOOLTIP" & RETURN & RETURN & "Generates a tool tip when the user rolls over the sprite." & RETURN & RETURN & "NOTE: This behavior calls the 'Display Text' behavior to actually show the message.  The 'Display Text' behavior must be attached to a different sprite which contains either a Field or a Text member." & RETURN & RETURN & "If such a sprite exists, it will automatically be selected in the Behavior Parameters dialog." & RETURN & RETURN & "If you wish the Tooltip to appear in a given position relative to the current sprite, choose the appropriate position in the Behavior Parameters dialog, and ensure that the associated 'Display Text' behavior is set to act as a Tooltip.  (If the 'Display Text' behavior is set to act as a Status Bar, then it will ignore any position data and appear in a fixed position)." & RETURN & RETURN & "You can choose to have the tooltip appear immediately on rollover, or to appear only if the mouse remains over the sprite for a given period.  You can also choose to have the tooltip disappear if the user clicks on the sprite." & RETURN & RETURN & "The Behavior Parameters dialog has limited space for entering a tool tip message.  In particular, it will not accept a string which contains the RETURN character.  If you need to display a long Tooltip which consists of several lines of text, and which must appear at the position of this sprite, then you must use send a message to this behavior containing the requiresd string.  For example:" & RETURN & RETURN & "  SendSprite (1, #Tooltip_SetMessage, " & QUOTE & "This message consists" & QUOTE & "&RETURN&" & QUOTE & "of two lines of text" & QUOTE & ")" & RETURN & RETURN & "This would produce the following message when the mouse rolls over sprite 1:" & RETURN & RETURN & "     This message consists" & RETURN & "     of two lines of text" & RETURN & RETURN & "If the tooltip generated by this behavior is to be diplayed in a Status bar then this step may be needed.  The 'Display Text' behavior will ensure that a long line of text is wrapped in the Status bar, and that scroll bars appear if necessary." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & "All" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Tool tip to display (single-line string)" & RETURN & "* Delay before displaying tool tip (0 - 2 seconds)" & RETURN & "* Hide tool tip if sprite is clicked? (TRUE | FALSE)" & RETURN & "* Position of tool tip relative to the sprite" & RETURN & "  (This will be ignored if the 'Display Text' behavior is set to act as a status bar)." & RETURN & "* Number of the sprite where tool tip is to be displayed." & RETURN & "  (This sprite should have the 'Display Text' behavior attached to it.  If the given sprite is moved an authortime alert will invite you to update the Behavior Parameters)." & RETURN & RETURN & "PUBLIC METHODS:" & RETURN & "=> Set the tooltip message (allows the RETURN character)" & RETURN & "=> Obtain behavior reference" & RETURN & RETURN & "ASSOCIATED BEHAVIORS:" & RETURN & "+ Display Text - ESSENTIAL - must be attached to a Field or Text sprite which covers the same span of frames." & RETURN & RETURN & "You can find the 'Display Text' behavior in the Library Palette, under Media > Text > Display Text."
end

on getBehaviorTooltip me
  return "Use with any type of member to generate a tool tip" & RETURN & "message when the mouse is over the sprite" & RETURN & RETURN & "This behavior requires that the 'Display Text' behavior" & RETURN & "be available on a Field or Text sprite to display the" & RETURN & "messages that it generates.  If no such sprite is" & RETURN & "available an alert will appear (authortime only)." & RETURN & RETURN & "The parameter set for the associated 'Display Text'" & RETURN & "behavior determines whether the tool tip message" & RETURN & "appears in a Status Bar or as a temporary Tooltip" & RETURN & "display over or near this sprite."
end

on beginSprite me
  Initialize(me)
end

on prepareFrame me
  CheckStatus(me)
end

on mouseEnter me
  myStartTicks = the ticks + myDelay
end

on mouseLeave me
  myStartTicks = the maxinteger
end

on Initialize me
  mySprite = sprite(me.spriteNum)
  myMember = mySprite.member
  if not voidp(getPDLError) then
    ErrorAlert(me, #getPDL_Invalid, myMember.type)
  end if
  myDisplayList = []
  myStartTicks = the maxinteger
end

on CheckStatus me
  if myStartTicks < the ticks then
    if myHideFlag then
      if the mouseDown then
        if myDisplayFlag then
          HideTip(me)
        end if
        exit
      end if
    end if
    if myDisplayFlag then
      exit
    end if
    ShowTip(me)
  else
    if myDisplayFlag then
      HideTip(me)
    end if
  end if
end

on ShowTip me
  myDisplayFlag = 1
  case myPosition of
    "centered above":
      theAlignment = #bottomCenter
      displayLoc = point((mySprite.left + mySprite.right) / 2, mySprite.top)
    "centered below":
      displayLoc = point((mySprite.left + mySprite.right) / 2, mySprite.bottom)
      theAlignment = #topCenter
    "at topLeft":
      displayLoc = point(mySprite.left, mySprite.top)
      theAlignment = #topLeft
    "at topRight":
      displayLoc = point(mySprite.right, mySprite.top)
      theAlignment = #topRight
    "centered":
      centerH = (mySprite.left + mySprite.right) / 2
      centerV = (mySprite.top + mySprite.bottom) / 2
      displayLoc = point(centerH, centerV)
      theAlignment = #center
    "at bottomLeft":
      displayLoc = point(mySprite.left, mySprite.bottom)
      theAlignment = #bottomLeft
    "at bottomRight":
      displayLoc = point(mySprite.right, mySprite.bottom)
      theAlignment = #bottomRight
    "at regPoint":
      displayLoc = mySprite.loc
      theAlignment = #center
    "under the mouse":
      displayLoc = the mouseLoc
      theAlignment = #bottomLeft
    #bottomLeft:
      displayLoc = point(mySprite.left, mySprite.bottom)
  end case
  if not myDisplayList.count() then
    EnrollDisplaySprite(me)
  end if
  call(#DisplayText_SetText, myDisplayList, myString, displayLoc, theAlignment)
end

on HideTip me
  myDisplayFlag = 0
  if not myDisplayList.count() then
    EnrollDisplaySprite(me)
  end if
  call(#DisplayText_SetText, myDisplayList, EMPTY)
end

on EnrollDisplaySprite me
  sendSprite(myDisplaySprite, #DisplayText_Enroll, myDisplayList)
  if not myDisplayList.count() then
    sendAllSprites(#DisplayText_Enroll, myDisplayList)
    if not myDisplayList.count() then
      ErrorAlert(me, #noValidSprites, myDisplaySprite)
    else
      ErrorAlert(me, #invalidSpriteNumber, myDisplaySprite)
    end if
  end if
end

on GetDisplaySprite me
  displayScriptMember = the number of member "Display Text"
  if displayScriptMember > 0 then
    displayScriptMember = member(displayScriptMember)
    theSprite = 1
    repeat while theSprite <= the lastChannel
      theScripts = sprite(theSprite).scriptList
      scriptCount = theScripts.count()
      repeat while scriptCount
        if theScripts[scriptCount][1] = displayScriptMember then
          return theSprite
        end if
        scriptCount = scriptCount - 1
      end repeat
      theSprite = 1 + theSprite
    end repeat
  end if
  return the currentSpriteNum + 1
end

on Tooltip_SetMessage me, theString
  case ilk(theString) of
    #string:
    otherwise:
      return #invalidTypeError
  end case
  myString = theString
end

on Tooltip_GetReference me
  return me
end

on ErrorAlert me, theError, Data
  behaviorName = string(me)
  delete word 1 of behaviorName
  delete char -30001 of behaviorName
  delete char -30001 of behaviorName
  case Data.ilk of
    #void:
      Data = "<void>"
    #symbol:
      Data = "#" & Data
  end case
  case theError of
    #invalidSpriteNumber:
      if the runMode = "Author" then
        alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & "Behavior " & behaviorName && RETURN & RETURN & "Sprite " & Data & " did not respond to a #DisplayText call.  Another sprite will be used.  Please open the Behavior Parameters dialog to choose the correct sprite for displaying the Tooltip message.")
      end if
    #noValidSprites:
      if the runMode = "Author" then
        alert("BEHAVIOR ERROR: Frame " & the frame & RETURN & "Behavior " & behaviorName && RETURN & RETURN & "No sprites responded to a #DisplayText call." & RETURN & RETURN & "Please ensure that the 'Display Text' behavior is attached to a Field or TextSprite in the same frames as Sprite " & me.spriteNum)
      end if
  end case
end

on getPropertyDescriptionList me
  displaySprite = GetDisplaySprite(me)
  return [#myString: [#comment: "Text of tool tip:", #format: #string, #default: "Insert your single-line tool tip here"], #myDelay: [#comment: "Pause before showing tool tip (ticks):", #format: #integer, #range: [#min: 0, #max: 120], #default: 30], #myHideFlag: [#comment: "Hide tool tip if user clicks on sprite?", #format: #boolean, #default: 1], #myPosition: [#comment: "Tool tip position relative to sprite (see notes):", #format: #string, #range: ["centered above", "at topLeft", "at topRight", "centered", "at bottomLeft", "at bottomRight", "centered below", "at regPoint", "under the mouse"], #default: "centered"], #myDisplaySprite: [#comment: "Use which sprite to display tooltip?", #format: #integer, #range: [#min: 1, #max: the lastChannel], #default: displaySprite]]
end
