property spriteNum, getPDLError, myDisplayType, mySprite, myMember, myWidthAdjust, myHeightAdjust, myOffStageLoc

on getBehaviorDescription me
  return "DISPLAY TEXT" & RETURN & RETURN & "This behavior allows you to display a given string in a Field or Text member.  Use it with the 'Tooltip' and 'Hypertext - Display Status' behaviors which need a Field or Text member in which to display their information.  Or create your own custom Lingo to display runtime information, such as the position of the mouse." & RETURN & RETURN & "This behavior waits for Lingo commands to tell it what to do.  It is not active by itself." & RETURN & RETURN & "You can choose between two display types: tooltip and status bar." & RETURN & RETURN & "The TOOLTIP type of display will make the Field or Text member resize itself to fit the text, and disappear when it is empty.  You can set the tooltip type display to appear at any position on the stage, for example: under the cursor.  If no position is sent to the sprite, it will appear at the top left corner of the Stage.  See the 'Tooltip' behavior for more details." & RETURN & RETURN & "If you wish to display several lines of text, you must use RETURN characters to define the line breaks.  An empty tooltip sprite will move off-stage to hide.  It is recommended to place it off-stage before it is used, in case it causes a brief flash on the screen." & RETURN & RETURN & "The STATUS BAR type of display will appear on Stage at all times.  It will not resize or change position.  Any positional information sent to this sprite will be ignored if it is set to act as a status bar.  If the text is too long to appear in the member of the current sprite, a scrollbar will appear.  You do not need to divide the text with RETURN characters.  If you think that a scrollbar may be necessary, make sure that the Field or Text member is sufficiently tall for the scroll arrows to operate correctly." & RETURN & RETURN & "Set the font size and other characteristics of Field or Text member to customize the appearance of the message." & RETURN & RETURN & "Be sure to give the Text or Field member a name.  It may be emptied by this behavior.  Director 7 automatically erases nameless empty members." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & "Field and Text members" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Display type:" & RETURN & "  - Tooltip (appears near the cursor on rollover)" & RETURN & "  - Status bar (appears in a fixed position at all times)" & RETURN & RETURN & "ASSOCIATED BEHAVIORS:" & RETURN & "+ Tooltip" & RETURN & "+ Source Status" & RETURN & "+ Hypertext - Display Status" & RETURN & RETURN & "PUBLIC METHODS:" & RETURN & "=> Set the text to display (and the position of the sprite)"
end

on getBehaviorTooltip me
  vTip = "Use with Field or Text members." & RETURN & RETURN
  vTip = vTip & "This behavior waits for a message from another" & RETURN
  vTip = vTip & "behavior or custom handler to display a character" & RETURN
  vTip = vTip & "string.  It is intended to be used with the 'Tooltip'," & RETURN
  vTip = vTip & "'Source Status' and 'Hypertext - Display Status' " & RETURN
  vTip = vTip & "behaviors to create a status bar or a tooltip " & RETURN
  vTip = vTip & "under the cursor."
  return vTip
end

on beginSprite me
  Initialize(me)
end

on endSprite me
  mySprite.visible = 1
end

on Initialize me
  mySprite = sprite(me.spriteNum)
  myMember = mySprite.member
  memberType = myMember.type
  case memberType of
    #field, #text:
    otherwise:
      return ErrorAlert(me, #invalidMemberType, memberType)
  end case
  myDisplayType = symbol(myDisplayType)
  if myMember.type = #field then
    myWidthAdjust = (myMember.margin + myMember.border) * 2
    myHeightAdjust = myMember.margin + (myMember.border * 2)
  else
    myWidthAdjust = 0
    myHeightAdjust = 0
  end if
  myMember.text = EMPTY
  if myDisplayType = #Tooltip then
    myMember.boxType = #fixed
    myOffStageLoc = point(999, 999)
    mySprite.loc = myOffStageLoc
  end if
end

on BestRect me, theString
  myMember.rect = rect(0, 0, 8000, 0)
  myMember.text = theString
  BestRect = myMember.rect
  theLine = the number of lines in theString
  theWidth = 0
  checkedChars = 0
  repeat while theLine
    endOfLine = offset(RETURN, theString)
    if not endOfLine then
      endOfLine = the number of chars in theString + 1
      myMember.text = myMember.text & RETURN
    end if
    checkedChars = checkedChars + endOfLine
    endPoint = charPosToLoc(myMember, checkedChars)
    lineWidth = endPoint[1]
    if lineWidth > theWidth then
      theWidth = lineWidth
    end if
    delete char 1 to endOfLine of theString
    theLine = theLine - 1
  end repeat
  lastChar = myMember.char.count
  lastCharLoc = charPosToLoc(myMember, lastChar)
  theHeight = lastCharLoc[2]
  BestRect[3] = theWidth + 1
  BestRect[4] = theHeight + 1
  return BestRect
end

on GetTopLeft me, theLoc, theAlignment, memberRect
  case theAlignment of
    #bottomCenter:
      return theLoc - [memberRect.width / 2, memberRect.height]
    #bottomRight:
      return theLoc - [memberRect.width, memberRect.height]
    #bottomLeft:
      return theLoc - [0, memberRect.height]
    #center:
      return theLoc - [memberRect.width / 2, memberRect.height / 2]
    #topCenter:
      return theLoc - [memberRect.width / 2, 0]
    #topRight:
      return theLoc - [memberRect.width, 0]
    otherwise:
      return theLoc
  end case
end

on DisplayText_Enroll me, enrollList
  if ilk(enrollList) <> #list then
    return me
  end if
  if not enrollList.count() then
    enrollList.append(me)
  else
  end if
  return enrollList
end

on DisplayText_SetText me, theString, theLoc, theAlignment
  if not stringp(theString) then
    ErrorAlert(me, #invalidString, theString)
    theString = string(theString)
  else
    case ilk(theLoc) of
      #void, #point:
      otherwise:
        ErrorAlert(me, #invalidPoint, theLoc)
        theLoc = point(0, 0)
    end case
  end if
  if (theString = EMPTY) and (myDisplayType = #Tooltip) then
    mySprite.loc = myOffStageLoc
  else
    myMember.text = theString
    if myDisplayType = #Tooltip then
      memberRect = BestRect(me, theString)
      myMember.rect = memberRect
    else
      memberRect = myMember.rect
    end if
    memberRect = memberRect + [0, 0, myWidthAdjust, myHeightAdjust]
    if myDisplayType = #Tooltip then
      if ilk(theLoc) <> #point then
        theLoc = point(0, 0)
      end if
      theLoc = GetTopLeft(me, theLoc, theAlignment, memberRect)
      stageWidth = (the activeWindow).rect.right - (the activeWindow).rect.left
      stageHeight = (the activeWindow).rect.bottom - (the activeWindow).rect.top
      maxH = stageWidth - memberRect.width
      maxV = stageHeight - memberRect.height
      theLoc[1] = max(0, min(theLoc[1], maxH))
      theLoc[2] = max(0, min(theLoc[2], maxV))
      theLoc = theLoc + myMember.regPoint
      mySprite.loc = theLoc
    else
      lastChar = theString.char.count
      textHeight = charPosToLoc(myMember, lastChar)[2]
      if textHeight > mySprite.height then
        myMember.boxType = #scroll
      else
        myMember.boxType = #fixed
      end if
    end if
  end if
end

on DisplayText_GetReference me
  return me
end

on ErrorAlert me, theError, Data
  case theError of
    #getPDLError:
      alert("Error: This behavior works only with Field and Text members." & RETURN & RETURN & "Hit OK and then delete this behavior from the sprite." & RETURN & RETURN & "For more information on deleting Behaviors, see the Help system.")
      if the optionDown then
        return [#getPDLError: [#comment: "ERROR:   Wrong member type.   Click 'Cancel'." & RETURN & "             Use only with Field and Text members.", #format: #string, #range: [EMPTY], #default: EMPTY]]
      end if
    otherwise:
      case theError of
        #invalidMemberType:
          behaviorName = string(me)
          delete word 1 of behaviorName
          delete char -30001 of behaviorName
          delete char -30001 of behaviorName
          alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & RETURN & "Behavior " & behaviorName & " only works with Field and Text members." & RETURN & RETURN & "Current member type = #" & Data)
          halt()
        #invalidString:
          if the runMode = "Author" then
            alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & "Behavior " & behaviorName & RETURN & RETURN & "The DisplayText_SetText handler could not treat the following as a string:" & RETURN & RETURN & Data)
          end if
        #invalidPoint:
          if the runMode = "Author" then
            alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & "Behavior " & behaviorName & RETURN & RETURN & "The DisplayText_SetText handler could not treat the following as a point:" & RETURN & RETURN & Data)
          end if
      end case
  end case
end

on getPropertyDescriptionList me
  if not (the currentSpriteNum) then
    exit
  end if
  theMember = sprite(the currentSpriteNum).member
  case theMember.type of
    #field, #text:
    otherwise:
      return ErrorAlert(me, #getPDLError)
  end case
  return [#myDisplayType: [#comment: "Display Text sprite behaves as a:", #format: #symbol, #range: ["status bar (fixed size and position)", "tooltip (dynamic size and position)"], #default: "in a fixed position (status bar)"]]
end
