property myHyperStyle, myMember, myMarkerList

on getBehaviorDescription me
  return "HYPERTEXT - GO TO MARKER" & RETURN & RETURN & "This behavior allows you to use Text member hyperlinks to jump to the various markers of the current movie." & RETURN & RETURN & "You can create links with the words" & RETURN & "  first" & RETURN & "  previous" & RETURN & "  next" & RETURN & "  last" & RETURN & "to navigate between markers, or you can use specific marker names." & RETURN & RETURN & "To create a hyperlink in a Text member, open the Text window and select the word(s) that you wish to use as the anchor.  Now open the Text Inspector and type the name of a marker.  When you run your movie, a click on the anchor text will move the playback head to the chosen marker." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & "Text members" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Use Hyperlink Styles?" & RETURN & "If you select this option, the anchor text will appear underlined and in blue.  A visited link will automatically appear in pink."
end

on getBehaviorTooltip me
  return "Use with Text members." & RETURN & RETURN & "Create hyperlinks to navigate to the" & RETURN & "various markers of the current movie." & RETURN & RETURN & "You can use specific marker names or" & RETURN & "the words 'first', 'previous', 'next'" & RETURN & "and 'last'."
end

on beginSprite me
  Initialize(me)
end

on hyperlinkClicked me, link, range
  JumpToMarker(me, link, range)
end

on Initialize me
  myMember = sprite(me.spriteNum).member
  memberType = myMember.type
  if memberType <> #text then
    ErrorAlert(me, #invalidMemberType, memberType)
  end if
  if myHyperStyle then
    myMember.useHypertextStyles = 1
  else
    myMember.useHypertextStyles = 0
  end if
  theLinks = myMember.hyperlinks
  linkCount = theLinks.count
  repeat while linkCount
    theRange = theLinks[linkCount]
    anchorRef = myMember.char[theRange[1]..theRange[2]].ref
    anchorRef.hyperlinkState = #normal
    linkCount = linkCount - 1
  end repeat
  myMarkerList = CreateMarkerList(me)
  SetToVisited(me)
end

on JumpToMarker me, link, range
  case link of
    "first":
      go(label(line 1 of the labelList))
    "next":
      go(#next)
    "prev", "previous":
      go(#previous)
    "last":
      go(marker(the maxinteger / 2))
    otherwise:
      linkValue = value(link)
      if (linkValue = link) or (link = ("(" & linkValue & ")")) or (link = "0") then
        linkValue = integer(linkValue)
        go(marker(linkValue))
      else
        theMarker = marker(link)
        if theMarker then
          go(theMarker)
        else
          ErrorAlert(me, #invalidMarker, link)
        end if
      end if
  end case
  SetToVisited(me)
end

on SetToVisited me
  hyperMarker = myMarkerList.findPos(marker(0))
  if hyperMarker then
    anchorChar = myMarkerList[hyperMarker]
    if anchorChar then
      myMember.char[anchorChar].hyperlinkState = #Visited
    end if
  end if
end

on CreateMarkerList me
  markerlist = [:]
  sort(markerlist)
  lastCheckedMarker = 0
  if marker(1) <> marker(-(the maxinteger) / 2) then
    i = 0
    repeat while i >= -(the maxinteger)
      checkMarker = marker(i)
      if checkMarker = lastCheckedMarker then
        exit repeat
      end if
      lastCheckedMarker = checkMarker
      markerlist.addProp(checkMarker, 0)
      i = -1 + i
    end repeat
  end if
  if marker(0) <> marker(the maxinteger / 2) then
    i = 1
    repeat while i <= the maxinteger
      checkMarker = marker(i)
      if checkMarker = lastCheckedMarker then
        exit repeat
      end if
      lastCheckedMarker = checkMarker
      markerlist.addProp(checkMarker, 0)
      i = 1 + i
    end repeat
  end if
  markerLinks = [:]
  theLinks = myMember.hyperlinks
  linkCount = theLinks.count
  i = 1
  repeat while i <= linkCount
    theRange = theLinks[i]
    theData = myMember.char[theRange[1]].hyperlink
    markerLinks.addProp(theData, theRange)
    i = 1 + i
  end repeat
  i = markerlist.count()
  theLabels = the labelList
  repeat while i
    markerName = theLabels.line[i]
    rangePos = markerLinks.findPos(markerName)
    if not rangePos then
      rangePos = markerLinks.count()
      repeat while rangePos
        if markerLinks.getPropAt(rangePos) = markerName then
          exit repeat
        end if
        rangePos = rangePos - 1
      end repeat
    end if
    if rangePos then
      markerlist[i] = markerLinks[rangePos][1]
    end if
    i = i - 1
  end repeat
  return markerlist
end

on ErrorAlert me, theError, Data
  case theError of
    #getPDLError:
      alert("Error: This behavior works only with Text members." & RETURN & RETURN & "Hit OK and then delete this behavior from the sprite." & RETURN & RETURN & "For more information on deleting Behaviors, see the Help system.")
      if the optionDown then
        return [#getPDLError: [#comment: "ERROR:   Wrong member type.   Click 'Cancel'." & RETURN & "                               Use only with Text members.", #format: #string, #range: [EMPTY], #default: EMPTY]]
      end if
    otherwise:
      behaviorName = string(me)
      delete word 1 of behaviorName
      delete char -30001 of behaviorName
      delete char -30001 of behaviorName
      case theError of
        #invalidMemberType:
          if the runMode = "Author" then
            alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & RETURN & "Behavior " & behaviorName & " only works with Text members." & RETURN & RETURN & "Current member type = #" & Data)
          end if
          abort()
        #invalidMarker:
          if the runMode = "Author" then
            alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & RETURN & "Behavior " & behaviorName & " could not find a marker called " & QUOTE & Data & QUOTE & ".")
          end if
      end case
  end case
end

on getPropertyDescriptionList me
  if not (the currentSpriteNum) then
    exit
  end if
  theMember = sprite(the currentSpriteNum).member
  if theMember.type <> #text then
    return ErrorAlert(me, #getPDLError)
  end if
  return [#myHyperStyle: [#comment: "Use the standard hyperlink style?", #format: #boolean, #default: 1]]
end
