property spriteNum, getPDLError, ourScrolledElement, myScrollRole, myActiveMember, myStandardMember, myMultiThreading, mySprite, myMember, myRect, myScrolledSprite, myScrolledMember, myActiveZone, myState, myNextUpdate, myZeroLoc, myDraggerShift, myClickV, myInterimShift, myMaxDraggerShift, myPageScroll, ourControlList, ourMaxScroll

on getBehaviorDescription me
  return "CUSTOM SCROLL BAR" & RETURN & RETURN & "Create dynamic scrollbars with your own artwork.  Such scrollbars are not Direct-To-Stage, so other sprites can appear over the top of them." & RETURN & RETURN & "You will need to create four graphic members:" & RETURN & "+ up arrow" & RETURN & "+ down arrow" & RETURN & "+ dragger" & RETURN & "+ backing bar" & RETURN & RETURN & "You may wish to use two additional members, to indicate that the arrow buttons have been pressed:" & RETURN & "+ up arrow (pressed state)" & RETURN & "+ down arrow (pressed state)" & RETURN & RETURN & "Place the four standard members on the Stage, and drop this behavior onto each of them.  Choose how the current sprite is to act in the appropriate pop-up menu in the Behavior Parameters dialog." & RETURN & RETURN & "For each element you can choose whether animations should continue in the background.  This option will tend to slow both the animations and the scrolling process, especially if applied to the arrow buttons." & RETURN & RETURN & "The various sprites will position themselves automatically to the right of the sprite-to-be-scrolled when the movie runs.  They revert to their original positions when it stops.  To avoid flashes, it would be a good idea to position them by hand in their intended positions." & RETURN & RETURN & "If you use a border on your field members, the scrollbar will sit outside the border.  Field box shadows are not recommended.  You can always fake external borders and box shadows with shape members (this even gives you a choice of colors)." & RETURN & RETURN & "To make authoring easier, this behavior will continue to work if you change EITHER the sprite channel OR the member in the chosen channel.  If you change both, the behavior will no longer know what to scroll." & RETURN & RETURN & "If you do change either the sprite or the member, and then reopen the Behavior Parameters dialog for one of the elements, this will put that behavior out of synch with the others.  Simply reopen and close the Parameters dialogs for each of the other elements.  If you do not do so, you will receive multiple alerts." & RETURN & RETURN & "This behavior can be used to scroll both editable and non-editable Fields and Text members.  For editable members, however, it does not automatically update the dragger position when the length of the text changes, nor does it make the editable member scroll automatically when the user drags the mouse to create a selection." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & PermittedMemberTypes(me) & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Current sprite acts as (up|down arrow | dragger | bar)" & RETURN & "* Scroll the member of <sprite>: <the member of sprite>" & RETURN & "* Standard member (this should not need to be set)" & RETURN & "* Member to display when arrow buttons are pressed" & RETURN & "* Allow animations to continue" & RETURN & RETURN & "PUBLIC METHODS" & RETURN & "=> Scroll the text/field member to a given position" & RETURN & "=> Swap the text/field  member to be scrolled" & RETURN & "=> Get the behavior reference"
end

on getBehaviorTooltip me
  return " Use 4 graphic members (dragger, bar, up and down arrows)" & RETURN & "to create a dynamic scrollbar for Text or Field members." & RETURN & RETURN & "Drop this behavior on each sprite separately." & RETURN & RETURN & "Sprites can pass over the scrollbar and animations can" & RETURN & "continue while the user scrolls." & RETURN & RETURN & "The behavior accepts Lingo calls to:" & RETURN & "+ set the scrolltop of the scrolled member to a given value." & RETURN & "+ swap the current member of the sprite."
end

on beginSprite me
  StartInstallation(me)
end

on prepareFrame me
  if not myState + 0 then
    FinishInstallation(me)
  end if
  if myMultiThreading then
    UpdateScroll(me)
  end if
end

on mouseDown me
  StartScroll(me)
end

on mouseUp me
  EndScroll(me)
end

on mouseUpOutSide me
  EndScroll(me)
end

on StartInstallation me
  mySprite = sprite(me.spriteNum)
  myMember = mySprite.member
  memberType = myMember.type
  if voidp(myScrollRole) then
    ErrorAlert(me, #getPDL_Invalid)
  end if
  myScrolledSprite = value(ourScrolledElement)
  myScrolledMember = myScrolledSprite.member
  memberType = myScrolledMember.type
  case memberType of
    #field, #text:
    otherwise:
      FindSprite(me)
  end case
  topLeft = point(myScrolledSprite.left, myScrolledSprite.top)
  myRect = myScrolledSprite.rect - rect(topLeft, topLeft)
  if memberType = #field then
    if myScrolledMember.border then
      myRect[1] = 1
    end if
  end if
  ourMaxScroll = []
  ourControlList = [:]
  sendAllSprites(#CustomScrollbar_RollCall, ourScrolledElement, ourControlList, ourMaxScroll)
  myState = 0
end

on FindSprite me
  saveDelimiter = the itemDelimiter
  the itemDelimiter = ":"
  memberData = ourScrolledElement.item[2]
  the itemDelimiter = saveDelimiter
  delete memberData.word[1]
  delete memberData.char[1]
  memberData = value(memberData)
  myScrolledMember = member(memberData)
  ScrollSprite = the lastChannel
  repeat while ScrollSprite
    if sprite(ScrollSprite).member = myScrolledMember then
      myScrolledSprite = sprite(ScrollSprite)
      exit
    end if
    ScrollSprite = ScrollSprite - 1
  end repeat
  if not ScrollSprite then
    ErrorAlert(me, #notScrollable, myScrolledSprite.member.type)
  end if
end

on FinishInstallation me
  CheckControlList(me)
  InstallElement(me)
end

on CheckControlList me
  checkList = duplicate(ourControlList)
  roleList = [#upArrow, #downArrow, #dragger, #bar]
  i = roleList.count()
  repeat while i
    theRole = roleList[i]
    rolePosition = checkList.findPos(theRole)
    if rolePosition then
      roleList.deleteAt(i)
      checkList.deleteAt(rolePosition)
    end if
    i = i - 1
  end repeat
  if checkList.count() then
    ErrorAlert(me, #extraControl, checkList.getPropAt(1))
  end if
  if roleList.count() then
    ErrorAlert(me, #missingControls, roleList)
  end if
end

on InstallElement me
  scrollRoof = myScrolledSprite.top
  scrollFloor = myScrolledSprite.bottom
  scrollLeft = myScrolledSprite.right
  if myScrolledMember.type = #field then
    scrollLeft = scrollLeft - (myScrolledMember.border <> 0)
  end if
  case myScrollRole of
    #upArrow:
      mySprite.loc = point(scrollLeft, scrollRoof) + myMember.regPoint
      myActiveZone = mySprite.rect
    #downArrow:
      locAdjust = myMember.regPoint - [0, mySprite.height]
      rectAdjust = mySprite.rect - rect(mySprite.loc, mySprite.loc)
      mySprite.loc = point(scrollLeft, scrollFloor) + locAdjust
      myActiveZone = rectAdjust + rect(mySprite.loc, mySprite.loc)
    #bar:
      barRoof = scrollRoof + call(#SpriteHeight, ourControlList.upArrow)
      barRight = scrollLeft + call(#spriteWIdth, ourControlList.upArrow)
      barFloor = scrollFloor - call(#SpriteHeight, ourControlList.downArrow)
      mySprite.rect = rect(scrollLeft, barRoof, barRight, barFloor)
    #dragger:
      case myMember.type of
        #vectorShape:
          regOffset = (myMember.strokeWidth + 1) / 2
          myMember.regPoint = point(regOffset, regOffset)
        otherwise:
          myMember.regPoint = point(0, 0)
      end case
      upHeight = call(#SpriteHeight, ourControlList.upArrow)
      upWidth = call(#SpriteHeight, ourControlList.upArrow)
      downHeight = call(#SpriteHeight, ourControlList.downArrow)
      draggerRoof = scrollRoof + upHeight
      myZeroLoc = point(scrollLeft, draggerRoof)
      barRight = scrollLeft + upWidth
      myActiveZone = rect(scrollLeft, scrollRoof, barRight, scrollFloor)
      myActiveZone = inflate(myActiveZone, 32, 32)
      arrowAdjust = upHeight + downHeight
      pageHeight = myScrolledSprite.height
      scrollHeight = pageHeight - arrowAdjust
      draggerHeight = mySprite.height
      myMaxDraggerShift = 0
      if scrollHeight < 0 then
        minRect = myScrolledMember.rect
        minRect[4] = arrowAdjust
        myScrolledMember.rect = minRect
        call(#InstallElement, ourControlList.bar)
      end if
      if scrollHeight < draggerHeight then
        mySprite.loc = point(-999, -999)
      else
        if myScrolledMember.type = #field then
          theMargin = myScrolledMember.margin
          theMargin = theMargin - (theMargin mod 2)
          borderAdjust = (myScrolledMember.border * 2) + theMargin
        else
          borderAdjust = 0
        end if
        myMaxDraggerShift = scrollHeight - draggerHeight + borderAdjust
      end if
      ourMaxScroll[1] = GetMaxScroll(me)
      SetDraggerShift(me)
  end case
  myState = #done
end

on GetMaxScroll me
  pageHeight = myScrolledSprite.height
  if myScrolledMember.type = #text then
    lastChar = myScrolledMember.char.count
    textHeight = charPosToLoc(myScrolledMember, lastChar)[2]
  else
    textHeight = myScrolledMember.height
  end if
  maxScroll = textHeight - pageHeight
  if maxScroll < 1 then
    return 0
  else
    return maxScroll
  end if
end

on StartScroll me
  myState = #Active
  ourMaxScroll[1] = GetMaxScroll(me)
  case myScrollRole of
    #dragger:
      myClickV = (the clickLoc)[2]
      myInterimShift = 0
    #upArrow, #downArrow:
      mySprite.member = myActiveMember
    #bar:
      draggerData = call(#GetDraggerData, ourControlList.dragger)
      if (the clickLoc)[2] < draggerData.top then
        myActiveZone = GetBarZone(me, draggerData, #up)
      else
        myActiveZone = GetBarZone(me, draggerData, #down)
      end if
  end case
  UpdateScroll(me)
  if myMultiThreading then
    exit
  end if
  repeat while the mouseDown
    UpdateScroll(me)
  end repeat
end

on UpdateScroll me
  case myState of
    #Active:
      if inside(the mouseLoc, myActiveZone) then
        case myScrollRole of
          #dragger:
            MoveDragger(me)
          #upArrow:
            move(me, 1)
          #downArrow:
            move(me, 0)
          #bar:
            MoveBar(me)
        end case
      else
        PauseScroll(me)
      end if
    #paused:
      ResumeScroll(me)
  end case
end

on GetBarZone me, draggerData, pageScroll
  if not voidp(pageScroll) then
    myPageScroll = pageScroll
  end if
  barZone = mySprite.rect
  if myPageScroll = #up then
    barZone[2] = myScrolledSprite.top
    barZone[4] = draggerData.top
  else
    barZone[2] = draggerData.bottom
    barZone[4] = myScrolledSprite.bottom
  end if
  return barZone
end

on MoveDragger me
  newScroll = myDraggerShift + the mouseV - myClickV
  newScroll = max(0, min(newScroll, myMaxDraggerShift))
  if myInterimShift = newScroll then
    exit
  end if
  myInterimShift = newScroll
  SetTextScroll(me, myInterimShift)
end

on move me, up
  if the ticks < myNextUpdate then
    exit
  end if
  if voidp(myNextUpdate) then
    myNextUpdate = the ticks + 10
  else
    myNextUpdate = the ticks + 1
  end if
  if up then
    if not myScrolledMember.scrollTop then
      exit
    end if
    scrollByLine(myScrolledMember, -1)
  else
    maxScroll = ourMaxScroll[1]
    if myScrolledMember.scrollTop = maxScroll then
      exit
    end if
    scrollByLine(myScrolledMember, 1)
    if myScrolledMember.scrollTop > maxScroll then
      myScrolledMember.scrollTop = maxScroll
    end if
  end if
  call(#SetDraggerShift, ourControlList.dragger)
end

on MoveBar me, up
  if the ticks < myNextUpdate then
    exit
  end if
  myNextUpdate = the ticks + 10
  if myPageScroll = #up then
    scrollByPage(myScrolledMember, -1)
    if myScrolledMember.scrollTop then
      scrollByLine(myScrolledMember, 1)
    end if
  else
    maxScroll = ourMaxScroll[1]
    if myScrolledMember.scrollTop = maxScroll then
      exit
    end if
    scrollByPage(myScrolledMember, 1)
    scrollByLine(myScrolledMember, -1)
    if myScrolledMember.scrollTop > maxScroll then
      myScrolledMember.scrollTop = maxScroll
    end if
  end if
  call(#SetDraggerShift, ourControlList.dragger)
  draggerData = call(#GetDraggerData, ourControlList.dragger)
  myActiveZone = GetBarZone(me, draggerData)
end

on PauseScroll me
  myState = #paused
  case myScrollRole of
    #dragger:
      SetTextScroll(me, myDraggerShift)
    #upArrow, #downArrow:
      mySprite.member = myStandardMember
      if not myMultiThreading then
        updateStage()
      end if
  end case
end

on ResumeScroll me
  myState = #Active
  case myScrollRole of
    #upArrow, #downArrow:
      mySprite.member = myActiveMember
  end case
  UpdateScroll(me)
end

on EndScroll me
  case myScrollRole of
    #dragger:
      if not voidp(myInterimShift) then
        if ourMaxScroll[1] then
          myDraggerShift = myInterimShift
        else
          myDraggerShift = 0
          ShiftDragger(me, myDraggerShift)
        end if
      end if
      myInterimShift = VOID
    #upArrow, #downArrow:
      mySprite.member = myStandardMember
      myNextUpdate = VOID
  end case
  myState = #done
end

on SetTextScroll me, draggerShift
  if ourMaxScroll[1] then
    textScroll = draggerShift * ourMaxScroll[1] / myMaxDraggerShift
    myScrolledMember.scrollTop = textScroll
  else
  end if
  ShiftDragger(me, draggerShift)
end

on SetDraggerShift me
  maxScroll = ourMaxScroll[1]
  if maxScroll then
    textScroll = myScrolledMember.scrollTop
    myDraggerShift = textScroll * myMaxDraggerShift / maxScroll
  else
    myDraggerShift = 0
  end if
  ShiftDragger(me, myDraggerShift)
end

on ShiftDragger me, draggerShift
  if not myMaxDraggerShift then
    exit
  end if
  mySprite.loc = myZeroLoc + [0, draggerShift]
  updateStage()
end

on CustomScrollbar_SetScroll me, theScroll
  case ilk(theScroll) of
    #integer:
    #float:
      theScroll = integer(theScroll)
    otherwise:
      return #invalidTypeError
  end case
  theScroll = max(0, min(theScroll, ourMaxScroll[1]))
  myScrolledMember.scrollTop = theScroll
  call(#SetDraggerShift, ourControlList.dragger, theScroll)
end

on CustomScrollbar_SwapMember me, newMember, currentMemberOrSprite
  case ilk(newMember) of
    #member:
    #integer, #string:
      memberExists = the number of member newMember > 0
      if memberExists then
        newMember = member(newMember)
      else
        return #memberInexistant
      end if
    otherwise:
      return #invalidType
  end case
  case newMember.type of
    #text, #field:
    otherwise:
      return #invalidMemberType
  end case
  case ilk(currentMemberOrSprite) of
    #sprite:
      if currentMemberOrSprite <> myScrolledSprite then
        exit
      end if
    #member:
      if currentMemberOrSprite <> myScrolledMember then
        exit
      end if
  end case
  myScrolledMember = newMember
  idealRect = myRect
  if myScrolledMember.type = #field then
    hBorder = myScrolledMember.border
    vBorder = hBorder - (hBorder <> 0)
    hMargin = myScrolledMember.margin
    vMargin = hMargin / 2
    idealRect = inflate(idealRect, -hBorder - hMargin, -hBorder - vMargin)
    if myRect[1] then
      if hBorder then
        idealRect[1] = idealRect[1] - 1
      end if
    else
      if hBorder then
        idealRect[1] = idealRect[1] - 1
      end if
    end if
  end if
  myScrolledMember.rect = idealRect
  myScrolledSprite.member = myScrolledMember
  myScrolledMember.boxType = #fixed
  i = ourControlList.count()
  repeat while i
    call(#newMember, ourControlList[i])
    i = i - 1
  end repeat
  updateStage()
  call(#InstallElement, ourControlList.dragger)
end

on CustomScrollbar_GetReference me, memberOrSprite, controlOrList
  case ilk(memberOrSprite) of
    #sprite:
      if memberOrSprite <> myScrolledSprite then
        exit
      end if
    #member:
      if memberOrSprite <> myScrolledMember then
        exit
      end if
    otherwise:
      exit
  end case
  if not voidp(controlOrList) then
    if ilk(controlOrList) = #propList then
      controlOrList.addProp(myScrollRole, me)
      return controlOrList
    else
      if controlOrList = myScrollRole then
        return me
      end if
    end if
  else
    return me
  end if
end

on CustomScrollbar_RollCall me, scrolledElement, controlList, maxScrollList
  if scrolledElement <> ourScrolledElement then
    exit
  end if
  ourControlList = controlList
  ourControlList.addProp(myScrollRole, me)
  ourMaxScroll = maxScrollList
end

on SpriteHeight me
  return mySprite.height
end

on spriteWIdth me
  return mySprite.width
end

on GetDraggerData me
  return [#top: mySprite.top, #bottom: mySprite.bottom]
end

on newMember me
  myScrolledMember = myScrolledSprite.member
end

on ErrorAlert me, theError, Data
  case theError of
    #getPDLError:
      permittedTypes = PermittedMemberTypes(me)
      alert("Error: This behavior works only with the following member types:  " & permittedTypes & RETURN & RETURN & "Hit OK and then delete this behavior from the sprite." & RETURN & "For more information on deleting Behaviors, see the Help system.")
      if the optionDown then
        return [#getPDLError: [#comment: "ERROR:  Click 'Cancel'. Wrong member type.", #format: #string, #range: [EMPTY], #default: EMPTY]]
      end if
    #noScrollableSprites:
      alert("Error: Please place a Field or Text member on the Stage before using this behavior to create a scroll bar." & RETURN & RETURN & "Hit OK and then delete this behavior from the sprite." & RETURN & "For more information on deleting Behaviors, see the Help system.")
      if the optionDown then
        return [#getPDLError: [#comment: "ERROR:   Click 'Cancel'.  Place a Text or Field" & RETURN & "member on Stage before creating scroll bar.", #format: #string, #range: [EMPTY], #default: EMPTY]]
      end if
    otherwise:
      behaviorName = string(me)
      delete word 1 of behaviorName
      delete char -30001 of behaviorName
      delete char -30001 of behaviorName
      case Data.ilk of
        #void:
          Data = "<void>"
        #symbol:
          Data = "#" & Data
      end case
      if theError <> #getPDL_Invalid then
        memberName = myScrolledMember.name
        if memberName = EMPTY then
          memberName = myScrolledMember
        else
          memberName = QUOTE & memberName & QUOTE
        end if
        memberName = myScrolledMember.type && memberName
      end if
      case theError of
        #getPDL_Invalid:
          alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & RETURN & "Parameters for the " & behaviorName & "behavior have not been set." & RETURN & RETURN & "Please reopen the Behavior Parameters dialog and choose again.")
          halt()
        #notScrollable:
          alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & "Behavior " & behaviorName & RETURN & RETURN & "Sprite " & myScrolledSprite.spriteNum & " does not contain a Field or Text member." & RETURN & "Choose again in the Behavior Parameters dialog." & RETURN & RETURN & "Member type = " & Data)
          halt()
        #extraControl:
          if the runMode = "Author" then
            alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & "Behavior " & behaviorName & RETURN & RETURN & "There is more than one " & Data & " sprite defined for the scroll bar for sprite " & myScrolledSprite.spriteNum & ", " & memberName & ".")
          end if
        #missingControls:
          alert("BEHAVIOR ERROR: Frame " & the frame & ", Sprite " & me.spriteNum & RETURN & "Behavior " & behaviorName & RETURN & RETURN & "The following elements of the scroll bar for sprite " & myScrolledSprite.spriteNum & ", " & memberName & " are missing:" & RETURN & RETURN & Data)
          halt()
      end case
  end case
end

on getPropertyDescriptionList me
  if not (the currentSpriteNum) then
    exit
  end if
  theMember = sprite(the currentSpriteNum).member
  memberType = theMember.type
  permittedTypes = PermittedMemberTypes(me)
  if not permittedTypes.getPos(memberType) then
    return ErrorAlert(me, #getPDLError, permittedTypes)
  end if
  scrollableList = GetScrollableSprites(me)
  if not scrollableList.count then
    return ErrorAlert(me, #noScrollableSprites)
  end if
  return [#myScrollRole: [#comment: "Current sprite acts as:", #format: #symbol, #range: [#upArrow, #downArrow, #dragger, #bar], #default: 1], #ourScrolledElement: [#comment: "Scroll the member of", #format: #string, #range: scrollableList, #default: scrollableList[1]], #myStandardMember: [#comment: "Standard member:", #format: #graphic, #default: theMember], #myActiveMember: [#comment: "(Arrows only) mouseDown member:", #format: #graphic, #default: theMember], #myMultiThreading: [#comment: "Allow animations to continue (slower)?", #format: #boolean, #default: 1]]
end

on GetScrollableSprites me, permittedTypes
  scrollableSprites = []
  theSprite = 1
  repeat while theSprite <= the lastChannel
    theMember = sprite(theSprite).member
    case theMember.type of
      #field, #text:
        memberName = theMember.name
        if memberName = EMPTY then
          memberName = theMember
        else
          memberName = QUOTE & memberName & QUOTE
        end if
        memberName = theMember.type && memberName
        scrollableSprites.append("sprite " & theSprite & ": " & memberName)
    end case
    theSprite = 1 + theSprite
  end repeat
  return scrollableSprites
end

on PermittedMemberTypes me
  return [#animGif, #bitmap, #filmLoop, #flash, #movie, #picture, #shape]
end
